import JavaBoolean = JVM.java$.lang$.Boolean;
import JavaInteger = JVM.java$.lang$.Integer;
import PacketEvent = JVM.net$.ccbluex$.liquidbounce$.event$.PacketEvent;
import Render2DEvent = JVM.net$.ccbluex$.liquidbounce$.event$.Render2DEvent;
import WorldEvent = JVM.net$.ccbluex$.liquidbounce$.event$.WorldEvent;
import BoolValue = JVM.net$.ccbluex$.liquidbounce$.value$.BoolValue;
import IntegerValue = JVM.net$.ccbluex$.liquidbounce$.value$.IntegerValue;
import ScaledResolution = JVM.net$.minecraft$.client$.gui$.ScaledResolution;
import ItemFood = JVM.net$.minecraft$.item$.ItemFood;
import S06PacketUpdateHealth = JVM.net$.minecraft$.network$.play$.server$.S06PacketUpdateHealth;
import GL11 = JVM.org$.lwjgl$.opengl$.GL11;

var scriptName = "FoodStatsHUD";
var scriptVersion = 0.2;
var scriptAuthor = "mumy++";

class FoodStatsHUD {

    private readonly setting = {
        boolean: (name: string, value: boolean, extend: object = {}) => {
            return new _AdaptedValue<boolean, JavaBoolean>(new (Java.extend(BoolValue, extend) as typeof BoolValue)(name, value));
        },
        integer: (name: string, value: number, minimun: number, maximun: number, extend: object = {}) => {
            return new _AdaptedValue<number, JavaInteger>(new (Java.extend(IntegerValue, extend) as typeof IntegerValue)(name, value, minimun, maximun));
        }
    };

    private readonly settings = {
        foodLevel: this.setting.boolean("FoodLevel", false),
        saturationLevel: this.setting.boolean("SaturationLevel", true),
        modifier: this.setting.boolean("Modifier", true, {
            isSupported: () => this.settings.foodLevel.get() || this.settings.saturationLevel.get()
        }),
        monitor: this.setting.boolean("Monitor", true, {
            isSupported: () => this.settings.foodLevel.get() || this.settings.saturationLevel.get()
        })
    };

    private packetFoodLevel = -1;
    private packetSaturationLevel = -1;

    public getName() {
        return "FoodStatsHUD";
    }

    public getDescription() {
        return "FoodStatsHUD-Module, by-mumy";
    }

    public getCategory() {
        return "Misc";
    }

    public onRender2D(event: Render2DEvent) {
        if (mc.thePlayer == null || !mc.playerController!.gameIsSurvivalOrAdventure()) {
            return;
        }
        this.drawLevel();
    }

    public onPacket(event: PacketEvent) {
        const packet = event.getPacket()!;
        if (packet instanceof S06PacketUpdateHealth) {
            this.packetFoodLevel = packet.getFoodLevel();
            this.packetSaturationLevel = packet.getSaturationLevel();
        }
    }

    public onWorld(event: WorldEvent) {
        this.packetFoodLevel = this.packetSaturationLevel = -1;
    }

    public addValues(values: _ValueAdapter) {
        for (let key of Object.keys(this.settings)) {
            values.add(this.settings[key as keyof typeof this.settings]);
        }
    }

    private drawLevel() {
        const food = this.settings.foodLevel.get();
        const saturation = this.settings.saturationLevel.get();
        if (!food && !saturation) {
            return;
        }
        const modifier = this.settings.modifier.get();
        const scale = new ScaledResolution(mc);
        const left = scale.getScaledWidth() / 2 + 10;
        const right = scale.getScaledWidth() / 2 + 91;
        const axis = scale.getScaledHeight();
        if (food) {
            const foodLevelAxis = axis - (saturation ? 36 : 35);
            if (modifier) {
                const foodModifier = this.getHeldItemFoodModifier();
                if (foodModifier > 0) {
                    const modifierLeft = Math.max(left, right - this.getLevelWidth(this.getFoodLevel() + foodModifier));
                    this.drawRect(modifierLeft, foodLevelAxis - 1, right, foodLevelAxis, 0x7EFF4600);
                    this.drawRect(modifierLeft, foodLevelAxis, right, foodLevelAxis + 1, 0x7EDC3700);
                }
            }
            const foodLevelLeft = Math.max(left, right - this.getLevelWidth(this.getFoodLevel()));
            this.drawRect(foodLevelLeft, foodLevelAxis - 1, right, foodLevelAxis, 0xFFFF4600);
            this.drawRect(foodLevelLeft, foodLevelAxis, right, foodLevelAxis + 1, 0xFFDC3700);
        } if (saturation) {
            const saturationLevelAxis = axis - (food ? 34 : 35);
            if (modifier) {
                const saturationModifier = this.getHeldItemSaturationModifier();
                if (saturationModifier > 0) {
                    const modifierLeft = Math.max(left, right - this.getLevelWidth(Math.min(this.getFoodLevel() + this.getHeldItemFoodModifier(), this.getSaturationLevel() + saturationModifier)));
                    this.drawRect(modifierLeft, saturationLevelAxis - 1, right, saturationLevelAxis, 0x7EFFC800);
                    this.drawRect(modifierLeft, saturationLevelAxis, right, saturationLevelAxis + 1, 0x7EC88C00);
                }
            }
            const saturationLevelLeft = Math.max(left, right - this.getLevelWidth(this.getSaturationLevel()));
            this.drawRect(saturationLevelLeft, saturationLevelAxis - 1, right, saturationLevelAxis, 0xFFFFC800);
            this.drawRect(saturationLevelLeft, saturationLevelAxis, right, saturationLevelAxis + 1, 0xFFC88C00);
        }
    }

    private getFoodLevel() {
        return !this.settings.monitor.get() || this.packetFoodLevel === -1 ? mc.thePlayer!.getFoodStats()!.getFoodLevel() : this.packetFoodLevel;
    }

    private getSaturationLevel() {
        return !this.settings.monitor.get() || this.packetSaturationLevel === -1 ? mc.thePlayer!.getFoodStats()!.getSaturationLevel() : this.packetSaturationLevel;
    }

    private getHeldItemFoodModifier() {
        const itemStack = mc.thePlayer!.getHeldItem();
        if (itemStack == null) {
            return 0;
        }
        const item = itemStack.getItem()!;
        if (item instanceof ItemFood) {
            return item.getHealAmount(itemStack);
        }
        return 0;
    }

    private getHeldItemSaturationModifier() {
        const itemStack = mc.thePlayer!.getHeldItem();
        if (itemStack == null) {
            return 0;
        }
        const item = itemStack.getItem()!;
        if (item instanceof ItemFood) {
            return item.getHealAmount(itemStack) * item.getSaturationModifier(itemStack) * 2;
        }
        return 0;
    }

    private getLevelWidth(level: number) {
        return Math.ceil(Math.max(0, Math.min(level, 20)) / 20 * 81);
    }

    private drawRect(paramXStart: number, paramYStart: number, paramXEnd: number, paramYEnd: number, color: number) {
        const alpha = (color >> 24 & 0xFF) / 255;
        const red = (color >> 16 & 0xFF) / 255;
        const green = (color >> 8 & 0xFF) / 255;
        const blue = (color & 0xFF) / 255;
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glDisable(GL11.GL_TEXTURE_2D);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        GL11.glEnable(GL11.GL_LINE_SMOOTH);
        GL11.glPushMatrix();
        GL11.glColor4f(red, green, blue, alpha);
        GL11.glBegin(GL11.GL_TRIANGLE_FAN);
        GL11.glVertex2d(paramXEnd, paramYStart);
        GL11.glVertex2d(paramXStart, paramYStart);
        GL11.glVertex2d(paramXStart, paramYEnd);
        GL11.glVertex2d(paramXEnd, paramYEnd);
        GL11.glEnd();
        GL11.glPopMatrix();
        GL11.glEnable(GL11.GL_TEXTURE_2D);
        GL11.glDisable(GL11.GL_BLEND);
        GL11.glDisable(GL11.GL_LINE_SMOOTH);
        GL11.glColor4f(1, 1, 1, 1);
    }

}

let scriptInstance: any;

function onLoad() {}

function onEnable() {
    scriptInstance = moduleManager.registerModule(new FoodStatsHUD());
}

function onDisable() {
    moduleManager.unregisterModule(scriptInstance);
}